<?php
/* $Id: liensSponsos.cls.php,v 1.6 2010/03/20 23:06:16 sarterm Exp $ */
include_once(dirname(__FILE__)."/config.inc.php");

/*
 * Stocke les informations sur une URL référante.
 * - referant : l'URL complète originale,
 * - hote : le nom d'hôte du serveur référant,
 * - protocole : le protocole correspondant à l'URL (http, ftp, etc...),
 * - baseUrl : l'URL racine du serveur référant,
 * - requete : le chemin d'accés sur le serveur référant,
 * - testMoteur : vaut true s'il a été vérifié si l'URL est un moteur de
 *                recherche,
 * - estMoteur : vaut true si l'URL provient d'un moteur de recherche,
 * - nomMoteur : le nom du moteur de recherche correspondant,
 * - varianteMoteur : la variante du moteur de recherche correspondant (images, blogs...),
 * - requeteMoteur : les termes de recherche qui ont mené à l'URL.
 */
class IBDLiensSponsos {
    private $iniFile = Array();

    /*
     * Constructeur de la classe.
     *
     * @param $iniFile Le chemin du fichier de mots clés à utiliser, en relatif
     *                 par rapport au répertoire contenant ce fichier PHP.
     */
    function IBDLiensSponsos($iniFile = "defaut.ini") {
        $this->init($iniFile);
    }
    
    /*
     * Ajoute les liens dans une portion de code HTML.
     *
     * @param $codeHTML   La portion de code à laquelle ajouter les liens.
     * @param $section    La section du fichier ini contenant les mots-clés à
     *                    traiter.
     * @param $nbMotsCles Le nombre maximum de mots clés à traiter dans la
     *                    portion de code.
     * @param $nbOccur    Le nombre maximum d'occurences à traiter pour un même
     *                    mot clé.
     * @return Le code HTML enrichi avec les liens.
     */
    public function ajouteLiens($codeHTML, $section = "", $nbMotsCles = 2, $nbOccur = 1) {
        if ($section == "") {
            foreach($this->iniFile as $section => $valeur) {
                $codeHTML = $this->ajouteLiens($codeHTML, $section, $nbMotsCles, $nbOccur);
            }
            return $codeHTML;
        }
        $motsCles = $this->iniFile[$section];
        $ajoutes = 0;
        $taille = sizeof($motsCles["cle"]);
        if ($nbMotsCles < 1) {
            $nbMotsCles = $taille;
        }
        $rand = rand(0, $taille - 1);
        for ($i = 0; $i < $taille; $i++) {
            preg_match_all('/[^a-z0-9\-]'.$motsCles["cle"][($i + $rand) % $taille].'[^a-z0-9\-]/i', $codeHTML, $matches, PREG_PATTERN_ORDER + PREG_OFFSET_CAPTURE);
            foreach($matches as $subMatches) {
                $occurences = Array();
                $positionBalises = $this->getPositionsBalisesInterdites($codeHTML);
                foreach($subMatches as $motCle) {
                    if ($motCle[0] == "") {
                        continue;
                    }
                    $debut = $motCle[1];
                    if ($this->isDansBalises($debut, $positionBalises)) { // le mot clé est dans une balise interdite, on l'ignore
                        continue;
                    }
                    $occurences[] = $motCle;
                }
                if (sizeof($occurences) > 0) { // des occurences ont été trouvées
                    $ajoutes++; // on incrément le compteur de mots clés trouvés
                    $codeHTML = $this->lierOccurences($codeHTML, $occurences, $motsCles["lien"][($i + $rand) % $taille], $nbOccur);
                    if ($ajoutes >= $nbMotsCles) { // le nombre de mots clés demandés a été trouvé, on sort
                        return $codeHTML;
                    }
                }
            }
        }
        return $codeHTML;
    }
    
    /*
     * Crée les liens sur les occurences d'un mot clé.
     *
     * @param $codeHTML   La portion de code à laquelle ajouter les liens.
     * @param $occurences Les descriptions (texte, position de début et de fin)
     *                    des différentes occurences du mot clé.
     * @param $lien       Le lien vers lequel les mots clés doivent pointer.
     * @param $nbOccur    Le nombre maximum d'occurences à traiter pour un même
     *                    mot clé.
     * @return Le code HTML enrichi avec les liens.
     */
    private function lierOccurences($codeHTML, $occurences, $lien, $nbOccur) {
        global $cfgIBDLS;
        if ($nbOccur < 1 || $nbOccur >= sizeof($occurences)) {
            $aLier = $occurences;
        }
        else if ($nbOccur == 1) {
            $aLier[] = $occurences[rand(0, sizeof($occurences) - 1)];
        }
        else {
            $randKeys = array_rand($occurences, $nbOccur);
            asort($randKeys);
            foreach($randKeys as $cle) {
                $aLier[] = $occurences[$cle];
            }
        }
        $decalage = 0;
        foreach($aLier as $occurence) {
            $lenCle = strlen($occurence[0]);
            $debut = $occurence[1] + $decalage + 1;
            $resultat = substr($codeHTML, 0, $debut);
            $aHref = "<a ".$cfgIBDLS["attributsLien"]." href=\"$lien\">";
            $resultat .= $aHref;
            $resultat .= substr($codeHTML, $debut, $lenCle - 2);
            $resultat .= "</a>";
            $resultat .= substr($codeHTML, $debut + $lenCle - 2);
            $decalage += strlen($aHref) + 4;
            $codeHTML = $resultat;
        }
        return $codeHTML;
    }
    
    /*
     * Détermine si une position donnée se situe à l'intérieur d'une des
     * balises interdites.
     * 
     * @param $position        La position à tester.
     * @param $positionBalises Les positions (début et fin) des balises
     *                         interdites.
     * @return true si la position est dans une balise, false sinon.
     */
    private function isDansBalises($position, $positionsBalises) {
        foreach ($positionsBalises as $positionBalise) {
            if ($position > $positionBalise["deb"] && $position < $positionBalise["fin"]) {$now = explode(' ',microtime());
                return true;
            }
        }
        return false;
    }
    
    /*
     * Détermine les positions (début et fin) des balises interdites.
     * 
     * @param $codeHTML Le code HTML à analyser.
     * @return Les positions des balises interdites.
     */
    private function getPositionsBalisesInterdites($codeHTML) {
        global $cfgIBDLS;
        $positions = Array();
        $i = 0;
        foreach ($cfgIBDLS["balisesInterdites"] as $pattern) {
            preg_match_all($pattern, $codeHTML, $matches, PREG_PATTERN_ORDER + PREG_OFFSET_CAPTURE);
            foreach($matches[0] as $balise) {
                $positions[$i]["deb"] = $balise[1];
                $positions[$i]["fin"] = $balise[1] + strlen($balise[0]);
                $i++;
            }
        }
        return $positions;
    }
    
    private function init($iniFile) {
        global $cfgIBDLS;
        $path = dirname(__FILE__)."/$iniFile";
        $fichier = basename($path);
        $cacheFile  = dirname(__FILE__)."/$fichier".md5($iniFile).".dat";
        clearstatcache();
        if ($cfgIBDLS["useCache"]
            && file_exists($cacheFile)
            && filemtime($path) < filemtime($cacheFile)) {
            $this->iniFile = unserialize(file_get_contents($cacheFile));
        }
        else {
            $this->iniFile = parse_ini_file($path, true);
            if ($cfgIBDLS["useCache"]) {
                file_put_contents($cacheFile, serialize($this->iniFile));
            }
        }
    }
    
}
?>